@@ -12,9 +12,9 @@ import pandas.util._test_decorators as td
 
 from pandas.core.dtypes.api import is_list_like
 
+import pandas as pd
 from pandas import DataFrame, Series
-import pandas.util.testing as tm
-from pandas.util.testing import assert_is_valid_plot_return_object, ensure_clean
+import pandas._testing as tm
 
 
 """
@@ -34,6 +34,7 @@ class TestPlotBase:
         self.mpl_ge_2_2_3 = compat._mpl_ge_2_2_3()
         self.mpl_ge_3_0_0 = compat._mpl_ge_3_0_0()
         self.mpl_ge_3_1_0 = compat._mpl_ge_3_1_0()
+        self.mpl_ge_3_2_0 = compat._mpl_ge_3_2_0()
 
         self.bp_n_objects = 7
         self.polycollection_factor = 2
@@ -103,6 +104,28 @@ class TestPlotBase:
             else:
                 assert ax.get_legend() is None
 
+    def _check_legend_marker(self, ax, expected_markers=None, visible=True):
+        """
+        Check ax has expected legend markers
+
+        Parameters
+        ----------
+        ax : matplotlib Axes object
+        expected_markers : list-like
+            expected legend markers
+        visible : bool
+            expected legend visibility. labels are checked only when visible is
+            True
+        """
+        if visible and (expected_markers is None):
+            raise ValueError("Markers must be specified when visible is True")
+        if visible:
+            handles, _ = ax.get_legend_handles_labels()
+            markers = [handle.get_marker() for handle in handles]
+            assert markers == expected_markers
+        else:
+            assert ax.get_legend() is None
+
     def _check_data(self, xp, rs):
         """
         Check each axes has identical lines
@@ -289,7 +312,7 @@ class TestPlotBase:
         axes : matplotlib Axes object, or its list-like
         xaxis : {'linear', 'log'}
             expected xaxis scale
-        yaxis :  {'linear', 'log'}
+        yaxis : {'linear', 'log'}
             expected yaxis scale
         """
         axes = self._flatten_visible(axes)
@@ -307,7 +330,7 @@ class TestPlotBase:
         axes_num : number
             expected number of axes. Unnecessary axes should be set to
             invisible.
-        layout :  tuple
+        layout : tuple
             expected layout, (expected number of rows , columns)
         figsize : tuple
             expected figsize. default is matplotlib default
@@ -514,20 +537,20 @@ def _check_plot_works(f, filterwarnings="always", **kwargs):
 
             plt.clf()
 
-            ax = kwargs.get("ax", fig.add_subplot(211))  # noqa
+            kwargs.get("ax", fig.add_subplot(211))
             ret = f(**kwargs)
 
-            assert_is_valid_plot_return_object(ret)
+            tm.assert_is_valid_plot_return_object(ret)
 
-            try:
-                kwargs["ax"] = fig.add_subplot(212)
-                ret = f(**kwargs)
-            except Exception:
-                pass
+            if f is pd.plotting.bootstrap_plot:
+                assert "ax" not in kwargs
             else:
-                assert_is_valid_plot_return_object(ret)
+                kwargs["ax"] = fig.add_subplot(212)
 
-            with ensure_clean(return_filelike=True) as path:
+            ret = f(**kwargs)
+            tm.assert_is_valid_plot_return_object(ret)
+
+            with tm.ensure_clean(return_filelike=True) as path:
                 plt.savefig(path)
         finally:
             tm.close(fig)
